@@ -715,7 +715,7 @@ func TestNilInterface(t *testing.T) {
 				}
 			`,
 			ExpectedErrors: []*gqlerrors.QueryError{
-				&gqlerrors.QueryError{
+				{
 					Message:       "x",
 					Path:          []interface{}{"b"},
 					ResolverError: errors.New("x"),
@@ -753,7 +753,7 @@ func TestErrorPropagationInLists(t *testing.T) {
 				null
 			`,
 			ExpectedErrors: []*gqlerrors.QueryError{
-				&gqlerrors.QueryError{
+				{
 					Message:       droidNotFoundError.Error(),
 					Path:          []interface{}{"findDroids", 1, "name"},
 					ResolverError: droidNotFoundError,
@@ -795,7 +795,7 @@ func TestErrorPropagationInLists(t *testing.T) {
 				}
 			`,
 			ExpectedErrors: []*gqlerrors.QueryError{
-				&gqlerrors.QueryError{
+				{
 					Message:       droidNotFoundError.Error(),
 					Path:          []interface{}{"findDroids", 1, "name"},
 					ResolverError: droidNotFoundError,
@@ -829,7 +829,7 @@ func TestErrorPropagationInLists(t *testing.T) {
 				}
 			`,
 			ExpectedErrors: []*gqlerrors.QueryError{
-				&gqlerrors.QueryError{
+				{
 					Message: `graphql: got nil for non-null "Droid"`,
 					Path:    []interface{}{"findNilDroids", 1},
 				},
@@ -906,7 +906,7 @@ func TestErrorPropagationInLists(t *testing.T) {
 				}
 			`,
 			ExpectedErrors: []*gqlerrors.QueryError{
-				&gqlerrors.QueryError{
+				{
 					Message:       quoteError.Error(),
 					ResolverError: quoteError,
 					Path:          []interface{}{"findDroids", 0, "quotes"},
@@ -941,12 +941,12 @@ func TestErrorPropagationInLists(t *testing.T) {
 				}
 			`,
 			ExpectedErrors: []*gqlerrors.QueryError{
-				&gqlerrors.QueryError{
+				{
 					Message:       quoteError.Error(),
 					ResolverError: quoteError,
 					Path:          []interface{}{"findNilDroids", 0, "quotes"},
 				},
-				&gqlerrors.QueryError{
+				{
 					Message: `graphql: got nil for non-null "Droid"`,
 					Path:    []interface{}{"findNilDroids", 1},
 				},
@@ -985,7 +985,7 @@ func TestErrorWithExtensions(t *testing.T) {
 				null
 			`,
 			ExpectedErrors: []*gqlerrors.QueryError{
-				&gqlerrors.QueryError{
+				{
 					Message:       droidNotFoundError.Error(),
 					Path:          []interface{}{"FindDroid"},
 					ResolverError: droidNotFoundError,
@@ -1021,7 +1021,7 @@ func TestErrorWithNoExtensions(t *testing.T) {
 				null
 			`,
 			ExpectedErrors: []*gqlerrors.QueryError{
-				&gqlerrors.QueryError{
+				{
 					Message:       err.Error(),
 					Path:          []interface{}{"DismissVader"},
 					ResolverError: err,
@@ -3074,10 +3074,20 @@ type inputArgumentsObjectMismatch2 struct{}
 
 type inputArgumentsObjectMismatch3 struct{}
 
+type fieldNameMismatch struct{}
+
 type helloInput struct {
 	Name string
 }
 
+type helloOutput struct {
+	Name string
+}
+
+func (*fieldNameMismatch) Hello() helloOutput {
+	return helloOutput{}
+}
+
 type helloInputMismatch struct {
 	World string
 }
@@ -3110,6 +3120,7 @@ func TestInputArguments_failSchemaParsing(t *testing.T) {
 	type args struct {
 		Resolver interface{}
 		Schema   string
+		Opts     []graphql.SchemaOpt
 	}
 	type want struct {
 		Error string
@@ -3217,6 +3228,21 @@ func TestInputArguments_failSchemaParsing(t *testing.T) {
 			},
 			Want: want{Error: "field \"Input\": *struct { Thing string } does not define field \"name\" (hint: missing `args struct { ... }` wrapper for field arguments, or missing field on input struct)\n\tused by (*graphql_test.inputArgumentsObjectMismatch3).Hello"},
 		},
+		"Struct field name inclusion": {
+			Args: args{
+				Resolver: &fieldNameMismatch{},
+				Opts:     []graphql.SchemaOpt{graphql.UseFieldResolvers()},
+				Schema: `
+					type Query {
+						hello(): HelloOutput!
+					}
+					type HelloOutput {
+						name: Int
+					}
+				`,
+			},
+			Want: want{Error: "string is not a pointer\n\tused by (graphql_test.helloOutput).Name\n\tused by (*graphql_test.fieldNameMismatch).Hello"},
+		},
 	}
 
 	for name, tt := range testTable {
@@ -3224,7 +3250,7 @@ func TestInputArguments_failSchemaParsing(t *testing.T) {
 		t.Run(name, func(t *testing.T) {
 			t.Parallel()
 
-			_, err := graphql.ParseSchema(tt.Args.Schema, tt.Args.Resolver)
+			_, err := graphql.ParseSchema(tt.Args.Schema, tt.Args.Resolver, tt.Args.Opts...)
 			if err == nil || err.Error() != tt.Want.Error {
 				t.Log("Schema parsing error mismatch")
 				t.Logf("got: %s", err)
@@ -4118,15 +4144,15 @@ func checkFieldTraces(t *testing.T, want, have []fieldTrace) {
 		t.Errorf("mismatched field traces: expected %d but got %d: %#v", len(want), len(have), have)
 	}
 
-	type comparsion struct {
+	type comparison struct {
 		want fieldTrace
 		have fieldTrace
 	}
 
-	m := map[string]comparsion{}
+	m := map[string]comparison{}
 
 	for _, ft := range want {
-		m[ft.fieldName] = comparsion{want: ft}
+		m[ft.fieldName] = comparison{want: ft}
 	}
 
 	for _, ft := range have {
@@ -4152,3 +4178,151 @@ func stringsEqual(want, have string) string {
 
 	return ""
 }
+
+type queryVarResolver struct{}
+type filterArgs struct {
+	Required string
+	Optional *string
+}
+type filterSearchResults struct {
+	Match *string
+}
+
+func (r *queryVarResolver) Search(ctx context.Context, args *struct{ Filter filterArgs }) []filterSearchResults {
+	return []filterSearchResults{}
+}
+
+func TestQueryVariablesValidation(t *testing.T) {
+	gqltesting.RunTests(t, []*gqltesting.Test{{
+		Schema: graphql.MustParseSchema(`
+			input SearchFilter {
+			  	required: String!
+			  	optional: String
+			}
+			
+			type SearchResults {
+				match: String
+			}
+			
+			type Query {
+				search(filter: SearchFilter!): [SearchResults!]!
+			}`, &queryVarResolver{}, graphql.UseFieldResolvers()),
+		Query: `
+        		query {
+        			search(filter: {}) {
+        				match
+        			}
+        		}`,
+		ExpectedErrors: []*gqlerrors.QueryError{{
+			Message:   "Argument \"filter\" has invalid value {}.\nIn field \"required\": Expected \"String!\", found null.",
+			Locations: []gqlerrors.Location{{Line: 3, Column: 27}},
+			Rule:      "ArgumentsOfCorrectType",
+		}},
+	}, {
+		Schema: graphql.MustParseSchema(`
+			input SearchFilter {
+				required: String!
+				optional: String
+			}
+			
+			type SearchResults {
+				match: String
+			}
+			
+			type Query {
+				search(filter: SearchFilter!): [SearchResults!]!
+			}`, &queryVarResolver{}, graphql.UseFieldResolvers()),
+		Query: `
+			query q($filter: SearchFilter!) {
+				search(filter: $filter) {
+					match
+				}
+			}`,
+		Variables: map[string]interface{}{"filter": map[string]interface{}{}},
+		ExpectedErrors: []*gqlerrors.QueryError{{
+			Message:   "Variable \"required\" has invalid value null.\nExpected type \"String!\", found null.",
+			Locations: []gqlerrors.Location{{Line: 3, Column: 5}},
+			Rule:      "VariablesOfCorrectType",
+		}},
+	}})
+}
+
+type interfaceImplementingInterfaceResolver struct{}
+type interfaceImplementingInterfaceExample struct {
+	A string
+	B string
+	C bool
+}
+
+func (r *interfaceImplementingInterfaceResolver) Hey() *interfaceImplementingInterfaceExample {
+	return &interfaceImplementingInterfaceExample{
+		A: "testing",
+		B: "test",
+		C: true,
+	}
+}
+
+func TestInterfaceImplementingInterface(t *testing.T) {
+	gqltesting.RunTests(t, []*gqltesting.Test{{
+		Schema: graphql.MustParseSchema(`
+        interface A {
+          a: String!
+        }
+        interface B implements A {
+          a: String!
+          b: String!
+        }
+        interface C implements B & A {
+          a: String!
+          b: String!
+          c: Boolean!
+        }
+        type ABC implements C {
+          a: String!
+          b: String!
+          c: Boolean!
+        }
+        type Query {
+          hey: ABC
+        }`, &interfaceImplementingInterfaceResolver{}, graphql.UseFieldResolvers(), graphql.UseFieldResolvers()),
+		Query: `query {hey { a b c }}`,
+		ExpectedResult: `
+				{
+					"hey": {
+						"a": "testing",
+						"b": "test",
+						"c": true
+					}
+				}
+			`,
+	}})
+}
+
+func TestCircularFragmentMaxDepth(t *testing.T) {
+	withMaxDepth := graphql.MustParseSchema(starwars.Schema, &starwars.Resolver{}, graphql.MaxDepth(2))
+	gqltesting.RunTests(t, []*gqltesting.Test{
+		{
+			Schema: withMaxDepth,
+			Query: `
+	              query {
+	                  ...X
+	              }
+
+	              fragment X on Query {
+	                  ...Y
+	              }
+	              fragment Y on Query {
+	                  ...X
+	              }
+	          `,
+			ExpectedErrors: []*gqlerrors.QueryError{{
+				Message: `Cannot spread fragment "X" within itself via Y.`,
+				Rule:    "NoFragmentCycles",
+				Locations: []gqlerrors.Location{
+					{Line: 7, Column: 20},
+					{Line: 10, Column: 20},
+				},
+			}},
+		},
+	})
+}
